// -*- C++ -*-
/*!
 * @file  USBCamera.cpp
 * @brief USB Camera component.
 * @date $Date$
 *
 * @author Author: AIST
 * URL: http://www.openrtm.org/openrtm/
 *
 * BSD
 *
 * $Id$
 */

#include "USBCamera.h"

// Module specification
// <rtc-template block="module_spec">
static const char* usbcamera_spec[] =
  {
    "implementation_id", "USBCamera",
    "type_name",         "USBCamera",
    "description",       "USB Camera component.",
    "version",           "1.0.0",
    "vendor",            "AIST",
    "category",          "ImageProcessing",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.deviceNumber", "0",
    // Widget
    "conf.__widget__.deviceNumber", "text",
    // Constraints
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
USBCamera::USBCamera(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_imageOut("image", m_image)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
USBCamera::~USBCamera()
{
}



RTC::ReturnCode_t USBCamera::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  addOutPort("image", m_imageOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("deviceNumber", m_deviceNumber, "0");
  
  // J̉摜Lv`NX̃CX^X𐶐
  cam = new ds_Camera();
  // </rtc-template>
  return RTC::RTC_OK;
}


RTC::ReturnCode_t USBCamera::onFinalize()
{
  // Lv`NX̃IuWFNg̔j
  delete cam;
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t USBCamera::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t USBCamera::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*!
 * Initialization of device
 */

RTC::ReturnCode_t USBCamera::onActivated(RTC::UniqueId ec_id)
{
  // Lv`NX̃IuWFNg̏
  if(cam->initialize())
	return RTC::RTC_OK;

  return RTC::RTC_ERROR;
}

/*!
 * Post-processing such as freeing allocated memory.
 */

RTC::ReturnCode_t USBCamera::onDeactivated(RTC::UniqueId ec_id)
{
  // Lv`NX̃IuWFNg̏I
  cam->finalize();
  return RTC::RTC_OK;
}

/*!
 * Capture images from the camera, and outputs the data from OutPort.
 */

RTC::ReturnCode_t USBCamera::onExecute(RTC::UniqueId ec_id)
{
  // J̉摜Lv`
  if (cam->capture() < 0)
	return RTC::RTC_OK;

  // 摜TCY̎擾
  int len = cam->getImageSize(); 
  CvSize size = cam->getSize();

  // AEg|[gϐ։摜Zbg
  m_image.pixels.length(len);
  m_image.width  = size.width;
  m_image.height = size.height;

  // AEg|[gϐ։摜f[^Zbg
  memcpy((void *)&(m_image.pixels[0]), cam->getImageData(), len);

  // AEg|[gf[^o
  m_imageOut.write();
  
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t USBCamera::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t USBCamera::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t USBCamera::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t USBCamera::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t USBCamera::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void USBCameraInit(RTC::Manager* manager)
  {
    coil::Properties profile(usbcamera_spec);
    manager->registerFactory(profile,
                             RTC::Create<USBCamera>,
                             RTC::Delete<USBCamera>);
  }
  
};


